/*-------------------------------------------------------------------+
SNS_LocalTime.mq4 
Copyright (c) 2009, Steven N. Scott 
mailto:steven.n.scott@gmail.com
---------------------------------------------------------------------+
VERSION HISTORY
v1.0 		- 	initial release											10/25/2009
v1.1		-  Added Market Open/Close Markers						10/26/2009
v1.2		-  modified to use SNS_ObjectLibrary.mqh				11/30/2009
----------------------------------------------------------------------

SNS_LOCALTIME
Allows you to set up a window wth your local time for each bar rather than the default (and otherwise un-changeable) MT4 Broker's time.

You have to tell the indicator what your hours offset FROM YOUR BROKER'S TIME is.

You can specify to display the time as 12-hour format instead of miltary 0-23.  When displayed in 12-hour format, AM or PM is displayed
beneath the time.  You can specific the colors and sizes for both the time and for the AM/PM marker.

You can also toggle on/off the display of market opens/closes for each of four markets: SYDNEY, TOKYO, LONDON, and NEW YORK.  You can
specify the color, width, and style of each line (open & close) and the font, size and color for each market independently.

TEXT ORIENTATION: By default, if you show market opening lines, they will be labelled with vertical text, but you can set the
TEXT.VERTICAL property to false if you would rather have the text written across the line instead.  When in HORIZONTAL mode, the "open" and 
"close" labels are written beneath each city name.

There are spacing adjustments available for tweaking as you select different fonts and font sizes to make it diplay as you wish.  
The LINE.SPACING property is only imporant in making sure the AM/PM marker is drawn far enough below the times when using HORIZONTAL TEXT.

The SESSION.OFFSETFROMDAILYLOW property allows you to adjust where the session open/close text is displayed (offset from the DAILY LOW).

The SESSION.LINESPACING property allows you to control the distance between the market name and the open/close text.

If you change Font Sizes from the defaults, you may have to tinker with the spacing and offset values to get it to look exactly
as you want it to.

*/

// THE FOLLWING INCLUDE (MQH) FILE NEEDS TO BE IN THE EXPERTS\INCLUDES FOLDER
#include <SNS_ObjectLibrary.mqh>

#property indicator_separate_window
#property indicator_maximum 10
#property indicator_minimum 0

/********************************************************************************************************************************************************
*********************************************************************************************************************************************************/

//____________________________________________________________________________________________________________________________________________________________
// INDICATOR CUSTOMIZABLE PROPERTIES

extern string 		InstanceName							=	"LocalTime";
extern int			Window									=  1;
extern int			Bars.ShowTimeFor						=	50; // To save time & memory - limit how many bars back to show time for
extern int			Time.OffsetFromBroker				=	-7;
extern bool			Time.12HourClock						=  true;

extern string		Time.Font								= 	"Tahoma Narrow";
extern int			Time.Font.Size							=  8;
extern color		Time.Font.Color						=	DarkGray;
extern string		AMPM.Font								= 	"Tahoma Narrow";
extern int			AMPM.Font.Size							=  10;
extern color		AMPM.Font.Color						=  DarkGoldenrod;

extern double		Time.SpecificOffset					=	25;
extern double		Line.Spacing							=  3.5;
extern bool			Text.Vertical							=  true;
extern string		opens.note1								=  "============================================";

extern int			SessionLabels.OffsetFromDayLow	=  20;
extern int			SessionLabels.LineSpacing			=	8;

extern string		Sydney.note								=  "--SYDNEY-----------------------------------";
extern bool			Sydney.Show								=  true;
extern string		Sydney.Annotate						=  "Sydney";
extern string		Sydney.Font								=  "Arial Bold";
extern int			Sydney.Font.Size						=  12;
extern color		Sydney.Font.Color						=  White;
extern string		Sydney.Open.Time						=  "04:00 PM";
extern color		Sydney.Open.Color						=	DarkGray;
extern int			Sydney.Open.Width						=  3;
extern int			Sydney.Open.Style						=	0;
extern string		Sydney.Close.Time						=  "01:00 AM";
extern color		Sydney.Close.Color					=	DimGray;
extern int			Sydney.Close.Width					=  1;
extern int			Sydney.Close.Style					=	1;

extern string		Tokyo.note								=  "--TOKYO------------------------------------";
extern bool			Tokyo.Show								=  true;
extern string		Tokyo.Annotate							=  "Tokyo";
extern string		Tokyo.Font								=  "Arial Bold";
extern int			Tokyo.Font.Size						=  12;
extern color		Tokyo.Font.Color						=  Chartreuse;
extern string		Tokyo.Open.Time						=  "06:00 PM";
extern color		Tokyo.Open.Color						=	Lime;
extern int			Tokyo.Open.Width						=  3;
extern int			Tokyo.Open.Style						=	0;
extern string		Tokyo.Close.Time						=  "03:00 AM";
extern color		Tokyo.Close.Color						=	Lime;
extern int			Tokyo.Close.Width						=  1;
extern int			Tokyo.Close.Style						=	1;

extern string		London.note								=  "--LONDON-----------------------------------";
extern bool			London.Show								=  true;
extern string		London.Annotate						=  "London";
extern string		London.Font								=  "Arial Bold";
extern int			London.Font.Size						=  12;
extern color		London.Font.Color						=  Orange;
extern string		London.Open.Time						=  "02:00 AM";
extern color		London.Open.Color						=	Yellow;
extern int			London.Open.Width						=  3;
extern int			London.Open.Style						=	0;
extern string		London.Close.Time						=  "11:00 AM";
extern color		London.Close.Color					=	DarkOrange;
extern int			London.Close.Width					=  1;
extern int			London.Close.Style					=	1;


extern string		NY.note									=  "--NEW YORK---------------------------------";
extern bool			NY.Show									=  true;
extern string		NY.Annotate								=  "NY";
extern string		NY.Font									=  "Arial Bold";
extern int			NY.Font.Size							=  12;
extern color		NY.Font.Color							=  Tomato;
extern string		NY.Open.Time							=  "07:00 AM";
extern color		NY.Open.Color							=	Red;
extern int			NY.Open.Width							=  3;
extern int			NY.Open.Style							=	0;
extern string		NY.Close.Time							=  "04:00 PM";
extern color		NY.Close.Color							=	Red;
extern int			NY.Close.Width							=  1;
extern int			NY.Close.Style							=	2;


//____________________________________________________________________________________________________________________________________________________________
//INTERNAL GLOBALS

string tag="LOCALTIME:";

string 	Indi.Prefix;
string 	Indi.Symbol;
int		Indi.Period;
string	Indi.TimeFrame;
int		Indi.Digits;
double	Indi.Point;


double TimeOffset, DayLow, Y.Inc;
string open="open",close="close";

//***************************************************************************************************************************************************************
//STANDARD INDICATOR FUNCTIONS


int init(){

   Indi.Symbol  		= Symbol();
   Indi.Period  		= Period() ;    
   Indi.TimeFrame		= Time.TFToString(Indi.Period) ;
   Indi.Prefix			= tag + "[" + Indi.TimeFrame + "]";
   Indi.Digits			=  Digits ;   
   Indi.Point			=  Point ;
   
   if(Indi.Digits == 5 || Indi.Digits == 3) { Indi.Digits = Indi.Digits - 1 ; Indi.Point = Indi.Point * 10 ; }   
   

	if(Time.SpecificOffset==0) TimeOffset=10*Indi.Point; else TimeOffset=Time.SpecificOffset*Indi.Point;
	
	Y.Inc=SessionLabels.LineSpacing*Indi.Point;
	
	IndicatorShortName("");
	
   return(0);

}

int deinit() {

   DeleteObjectsByPrefix(Indi.Prefix);
   return(0);
   
}

int start(){

   deinit();
   DayLow=iLow(Indi.Symbol,PERIOD_D1,0);
   
   if(IndicatorCounted()<0) return(-1);
   
	ShowTimes();
	
   return(0);
   
}


//***************************************************************************************************************************************************************
//CUSTOM INDICATOR FUNCTIONS

void ShowTimes() {
datetime Time.Local,itime;
string Bar.Time,hs,ms,ap,ft;
int h;

	for(int i=Bars.ShowTimeFor; i>=0; i--) {
	
		itime=iTime(Indi.Symbol,Indi.Period,i);
		Time.Local = Time.AddPeriod(itime,PERIOD_H1,Time.OffsetFromBroker);
		h=TimeHour(Time.Local);
		if (Time.12HourClock) {
		
			if (h==0) { h=12; ap="AM";} else if (h==12) ap="PM"; else if(h>12) { h=h-12; ap="PM"; } else ap="AM";
			
		}
		hs=""+h; if (StringLen(hs)<2) hs="0"+hs;
		ms=TimeMinute(Time.Local); if (StringLen(ms)<2) ms="0"+ms;
		Bar.Time = hs+":"+ms;
		Draw_Text(Indi.Prefix, " "+i+" ("+Bar.Time+")", Bar.Time, itime, 9, Time.Font, Time.Font.Size, Time.Font.Color, false, Window);
		if (Time.12HourClock) Draw_Text(Indi.Prefix, " "+i+" ("+Bar.Time+"ap)",ap, itime, 9-Line.Spacing, AMPM.Font, AMPM.Font.Size, AMPM.Font.Color, false, Window);
		//Draw_VText(Indi.Prefix, " "+i+" ("+Bar.Time+")", Bar.Time, itime, 9, Time.Font, Time.Font.Size, Time.Font.Color, false, Window);

		ft=Bar.Time+" "+ap;
		if (Sydney.Show) {
		
			if (Sydney.Open.Time==ft) ShowTime("SYD.OPEN", Sydney.Annotate, open,Sydney.Open.Style, Sydney.Open.Width, Sydney.Open.Color, Sydney.Font, Sydney.Font.Size, Sydney.Font.Color,i);
			if (Sydney.Close.Time==ft) ShowTime("SYD.CLOSE", Sydney.Annotate, close,Sydney.Close.Style, Sydney.Close.Width, Sydney.Close.Color, Sydney.Font, Sydney.Font.Size, Sydney.Font.Color,i);
		
		}
		if (Tokyo.Show) {
		
			if (Tokyo.Open.Time==ft) ShowTime("TOK.OPEN", Tokyo.Annotate, open, Tokyo.Open.Style, Tokyo.Open.Width, Tokyo.Open.Color, Tokyo.Font, Tokyo.Font.Size, Tokyo.Font.Color,i);
			if (Tokyo.Close.Time==ft) ShowTime("TOK.CLOSE", Tokyo.Annotate, close, Tokyo.Close.Style, Tokyo.Close.Width, Tokyo.Close.Color, Tokyo.Font, Tokyo.Font.Size, Tokyo.Font.Color,i);
		
		}		
		if (London.Show) {
		
			if (London.Open.Time==ft) ShowTime("LON.OPEN", London.Annotate, open,London.Open.Style, London.Open.Width, London.Open.Color, London.Font, London.Font.Size, London.Font.Color,i);
			if (London.Close.Time==ft) ShowTime("LON.CLOSE", London.Annotate, close, London.Close.Style, London.Close.Width, London.Close.Color, London.Font, London.Font.Size, London.Font.Color,i);
		
		}		
		if (NY.Show) {
		
			if (NY.Open.Time==ft) ShowTime("NY.OPEN", NY.Annotate, open, NY.Open.Style, NY.Open.Width, NY.Open.Color, NY.Font, NY.Font.Size, NY.Font.Color,i);
			if (NY.Close.Time==ft) ShowTime("NY.CLOSE", NY.Annotate, close, NY.Close.Style, NY.Close.Width, NY.Close.Color, NY.Font, NY.Font.Size, NY.Font.Color,i);
		
		}	
		
	}


}

//____________________________________________________________________________________________________________________________________________________________

void ShowTime(string name,string abrv, string which, int style, int width, color clr, string font, int size, color fontcolor, int i){

	datetime x=iTime(Indi.Symbol,Indi.Period,i);
	double y=iLow(Indi.Symbol,PERIOD_D1,0)-(SessionLabels.OffsetFromDayLow*Indi.Point);

	Draw_VLine(Indi.Prefix, name, style, width, clr, x, true, 0);

	if (Text.Vertical) {
	
    	Draw_VText(Indi.Prefix, name+"lbl", abrv + " " + which, x,y, font, size, fontcolor, false,0);	
	
	} else {
	
		if (which==close) y=y-(2.5*Y.Inc);
   	Draw_Text(Indi.Prefix, name+"lbl", abrv, x,y, font, size, fontcolor, false,0);
   	Draw_Text(Indi.Prefix, name+"lbl2", which, x,y-Y.Inc, font, size-2, fontcolor, false,0);
	
	}
	
}